const UsuariosModel = require('../models/Usuarios.js');
const TipoUsuarioModel = require('../models/TipoUsuarios.js');
const DespachoModel = require('../models/Despachos.js');
const productoModel = require('../models/Productos.js');
const SuscripcionesModel = require('../models/Suscripciones.js');
const VentasSchema = require('../models/Ventas.js');
const ModulosSchema = require('../models/Modulos.js');
const TipoUsuario = require('../models/TipoUsuarios.js');
const ClienteModel = require('../models/Clientes.js');
const moment = require('moment-timezone');
const fs = require('fs');
const path = require('path');
const APP_URL = process.env.APP_URL;
const { uploadFile, deleteFileS3 } = require('../config/s3.js');
const {
  validarPassword,
  generatePassword,
  encriptar,
  desencriptar,
  sendWhatsappRecovery
} = require('../config/functionsGlobal.js');
const { sendMail } = require('../config/mail.js');
const { RegistroUsuarioHTML } = require('../Mail/RegistroUsuarioHTML.js');
const DESPACHO_APP = process.env.DESPACHO_APP;
const CLIENTE_APP = process.env.CLIENTE_APP;
const DISTRIBUIDOR_APP = process.env.DISTRIBUIDOR_APP;
const CuentasBancariasModel = require('../models/CuentaBancaria.js');
const { buildFileUri } = require('../config/s3.js');
const Ventas = require('../models/Ventas.js');

const login = async (req, res) => {
  try {
    const { correo, password, tipo } = req.body;

    // Validación inicial
    if (!correo || !password || !tipo) {
      return res
        .status(400)
        .json({ message: 'Correo y contraseña son requeridos' });
    }

    // Buscar el usuario en la base de datos
    const userFind = await UsuariosModel.findOne({
      email: correo,
      estatus: 'Activo',
      tipo,
      password: encriptar(password)
    }).populate('despacho');

    if (!userFind) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    // // Validar la contraseña
    // const passwordValid = validarPassword(password, userFind.password);
    // if (!passwordValid) {
    //   return res.status(401).json({ message: 'Contraseña incorrecta' });
    // }

    // Construir URIs para las imágenes
    if (userFind.foto) {
      userFind.foto = buildFileUri(
        `despachos/${userFind?.despacho?._id}/usuarios/${userFind.foto}`
      );
    } else {
      userFind.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }

    if (userFind.despacho && userFind.despacho.logo) {
      userFind.despacho.logo = `${APP_URL}/uploads/despachos/${userFind.despacho.logo}`;
    }

    // Responder con el usuario autenticado
    res.status(200).json({ message: 'Usuario logeado', data: userFind });
  } catch (error) {
    console.error(error);
    res
      .status(500)
      .json({ message: 'Error del servidor', error: error.message });
  }
};

const loginCliente = async (req, res) => {
  try {
    const { correo, password } = req.body;

    if (!correo || !password) {
      return res.status(400).json({ message: 'Faltan datos' });
    }

    const userFind = await ClienteModel.findOne({ correo, estatus: 'Activo' });

    if (!userFind) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    const passwordValid = validarPassword(password, userFind.password);

    if (!passwordValid) {
      return res.status(404).json({ message: 'Contraseña incorrecta' });
    }

    userFind.password = desencriptar(userFind.password);

    console.log(path.join('src/uploads/clientes', userFind.foto));
    if (
      userFind.foto !== '' &&
      fs.existsSync(path.join('src/uploads/clientes', userFind.foto))
    ) {
      userFind.foto = `${APP_URL}/uploads/clientes/${userFind.foto}`;
    } else {
      userFind.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }

    res.status(200).json({ message: 'Usuario logeado', data: userFind });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const register = async (req, res) => {
  try {
    const fecha = new Date();
    const {
      nombre,
      apellidoPaterno = '',
      apellidoMaterno = '',
      telefono = '',
      email = '',
      tipoUsuario = '',
      producto,
      formaPago,
      estado
    } = req.body;

    if (tipoUsuario === 'despacho') {
      if (!producto || !formaPago) {
        return res
          .status(400)
          .json({ message: 'Falta el producto o forma de pago' });
      }
    }

    let objDespacho = {}; // ? objeto para guardar los datos del despacho
    let precioProducto = 0; // ? precio del producto

    const objUser = {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email
    };

    if (!nombre || !email || !tipoUsuario) {
      return res.status(400).json({ message: 'Faltan datos' });
    }

    const findEmailExist = await UsuariosModel.findOne({
      email
    });

    if (findEmailExist) {
      return res.status(400).json({ message: 'El correo ya existe' });
    }
    // ? buscamos el id del tipo de usuario que se esta registrando
    if (tipoUsuario === 'admin') {
      objUser.tipo = 'admin';
      const findTipoUsuarioAdmin = await TipoUsuarioModel.findOne({
        tipo: 'admin'
      });
      // ? si no existe el tipo de usuario admin lo creamos

      if (!findTipoUsuarioAdmin) {
        return res
          .status(400)
          .json({ message: 'No existe el tipo de usuario admin' });
      } else {
        objUser.tipoUsuario = findTipoUsuarioAdmin._id;
      }
    } else if (tipoUsuario === 'despacho') {
      // ? buscamos el id del tipo de usuario que se esta registrando
      objUser.tipo = 'despacho';

      const findTipoUsuarioDespacho = await TipoUsuarioModel.findOne({
        tipo: 'despacho'
      });

      if (!findTipoUsuarioDespacho) {
        return res
          .status(400)
          .json({ message: 'No existe el tipo de usuario despacho' });
      } else {
        const findProducto = await productoModel.findById(producto);

        if (!findProducto) {
          return res.status(400).json({ message: 'No existe el producto' });
        }

        const { precio, cantidad } = findProducto; // ? obtenemos el precio y la cantidad del producto
        precioProducto = precio;
        // ? Vigencia de la cuenta un mes mas a la fecha actual
        const vigencia = fecha.setMonth(fecha.getMonth() + 1);

        const contadorExp = {
          contador: 0,
          limite: cantidad,
          vigencia
        };

        objDespacho = {
          contadorExp,
          correo: email,
          telefono,
          estado
        };

        objUser.tipoUsuario = findTipoUsuarioDespacho._id;
      }
    }

    const passwordGenerate = generatePassword();

    const passwordEncrypt = encriptar(passwordGenerate);

    objUser.password = passwordEncrypt;

    const userSave = await UsuariosModel.create(objUser);

    const htmlRegistro = RegistroUsuarioHTML(nombre, email, passwordGenerate);

    sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', email);

    if (tipoUsuario === 'despacho') {
      // ? si el usuario es de tipo despacho guardamos los datos del despacho
      objDespacho.creadoPor = userSave._id;

      const despachoCreate = await DespachoModel.create(objDespacho);

      const suscripcion = {
        // ? creamos el objeto de la suscripción
        despacho: despachoCreate._id,
        fechaFin: fecha.setMonth(fecha.getMonth() + 1),
        fechaInicio: fecha,
        estatus: 'Vigente',
        creadoPor: userSave._id,
        producto, // ? id del producto
        precio: precioProducto,
        observaciones: 'Se adquirió el producto por primera vez'
      };

      await SuscripcionesModel.create(suscripcion);

      const userDespacho = await UsuariosModel.findByIdAndUpdate(userSave._id, {
        despacho: despachoCreate._id
      });

      const venta = {
        despacho: despachoCreate._id,
        fechaVenta: fecha,
        producto,
        importe: precioProducto,
        fecha_pago: fecha,
        formaPago,
        estatus: 'Pagado'
      };

      await VentasSchema.create(venta);

      const findModulos = await ModulosSchema.find({
        estatus: 'Activo',
        tipo: 'despacho'
      });

      const objTipoUsuario = {
        nombre: 'Administrador',
        tipo: 'despacho',
        despacho: despachoCreate._id,
        modulos: findModulos.map((modulo) => {
          return {
            modulo: modulo._id,
            permisos: {
              create: true,
              read: true,
              update: true,
              delete: true,
              download: true
            }
          };
        })
      };

      await TipoUsuario.create(objTipoUsuario);

      // generar modulos para el despacho

      res.status(201).json({
        message: 'Usuario creado',
        data: userDespacho,
        despacho: despachoCreate
      });
    } else {
      res.status(201).json({ message: 'Usuario creado', data: userSave });
    }
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const rememberPassword = async (req, res) => {
  try {
    const { correo, tipo, acceso = 'despacho' } = req.body;

    if (!correo) {
      return res.status(400).json({ message: 'Falta el correo' });
    }

    if (acceso === 'cliente') {
      const findCliente = await ClienteModel.findOne({
        correo,
        estatus: 'Activo'
      });

      if (!findCliente) {
        return res.status(404).json({ message: 'Cliente no encontrado' });
      }

      if (!findCliente.password) {
        // generar contraseña aleatoria
        const passwordSin = generatePassword();
        const passwordEncrypt = encriptar(passwordSin);
        findCliente.password = passwordEncrypt;

        await findCliente.save();
      }

      const password = desencriptar(findCliente.password);

      const nombreCompleto = `${findCliente.nombre}`;
      const htmlRegistro = RegistroUsuarioHTML(
        nombreCompleto,
        correo,
        password,
        CLIENTE_APP
      );

      if (findCliente.telefono && tipo === 'whatsapp') {
        // sendWhatsappRecovery({ to: findCliente.telefono, nombre: nombreCompleto, correo, password, url: CLIENTE_APP, message });
      }

      if (findCliente.correo && tipo === 'email') {
        sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);
      }

      if (tipo === 'todos') {
        if (findCliente.telefono) {
          sendWhatsappRecovery({
            to: findCliente.telefono,
            correo,
            password,
            url: CLIENTE_APP
          });
        }

        if (findCliente.correo) {
          sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);
        }
      }

      findCliente.password = password;

      const messageWP = `Hola *${findCliente.nombre}* te enviamos tus datos de acceso a la plataforma:

Correo electrónico: ${correo}

Contraseña: ${password}

Para acceder a la plataforma, solo haz clic aquí ${CLIENTE_APP}`;
      return res
        .status(200)
        .json({ message: 'Accesos enviados', tipo: 'cliente', messageWP });
    } else {
      // ? si el acceso es para el despach o admin
      const findUser = await UsuariosModel.findOne({
        email: correo,
        estatus: 'Activo'
      });
      if (!findUser) {
        if (!findUser) {
          return res.status(404).json({ message: 'Usuario no encontrado' });
        }
      }

      const password = desencriptar(findUser.password);

      const nombreCompleto = `${findUser.nombre} ${findUser.apellidoPaterno} ${findUser.apellidoMaterno}`;
      const htmlRegistro = RegistroUsuarioHTML(
        nombreCompleto,
        correo,
        password
      );

      if (findUser.telefono && tipo === 'whatsapp') {
        // sendWhatsappRecovery({ to: findUser.telefono, nombre: nombreCompleto, correo, password, url: DESPACHO_APP, message });
      }

      if (findUser.email && tipo === 'email') {
        sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);
      }

      if (tipo === 'todos') {
        if (findUser.telefono) {
          sendWhatsappRecovery({
            to: findUser.telefono,
            correo,
            password,
            url: DESPACHO_APP
          });
        }

        if (findUser.email) {
          sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);
        }
      }

      findUser.password = password;

      const messageWP = `Hola *${findUser.nombre}*, te enviamos tus datos de acceso a la plataforma IUSFORM:

Correo electrónico: ${findUser.email}

Contraseña: ${password}

Para acceder a la plataforma, solo haz clic aquí ${DESPACHO_APP}`;

      res
        .status(200)
        .json({ message: 'Accesos enviados', tipo: 'despacho', messageWP });
    }
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const actualizarUsuario = async (req, res) => {
  try {
    const { id } = req.params;

    // const filename = req?.file?.filename ?? null;

    const {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email,
      password
    } = req.body;

    const passwordEncrypt = encriptar(password);

    // Actualizar el usuario sin devolver el objeto actualizado
    const objAntes = await UsuariosModel.findById(id);

    if (!objAntes) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    if (req.files && req.files.foto) {
      deleteFileS3(`despacho/${objAntes._id}/usuarios/${objAntes.foto}`);
    }

    objAntes.nombre = nombre ?? objAntes.nombre;
    objAntes.apellidoPaterno = apellidoPaterno ?? objAntes.apellidoPaterno;
    objAntes.apellidoMaterno = apellidoMaterno ?? objAntes.apellidoMaterno;
    objAntes.telefono = telefono ?? objAntes.telefono;
    objAntes.email = email ?? objAntes.email;
    objAntes.password = passwordEncrypt ?? objAntes.password;

    if (req.files && req.files.foto) {
      const foto = await uploadFile(
        req.files.foto,
        `despachos/${objAntes.despacho}/usuarios`
      );
      objAntes.foto = foto.fileName;
    }

    await objAntes.save();

    // Buscar y devolver el objeto actualizado
    const userUpdate = await UsuariosModel.findById(id).populate('despacho');

    if (userUpdate.foto) {
      userUpdate.foto = buildFileUri(
        `despachos/${userUpdate?.despacho?._id}/usuarios/${userUpdate.foto}`
      );
    } else {
      userUpdate.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }

    if (userUpdate.despacho && userUpdate.despacho.logo) {
      userUpdate.despacho.logo = `${APP_URL}/uploads/despachos/${userUpdate.despacho.logo}`;
    }

    userUpdate.password = desencriptar(userUpdate.password);

    res.status(200).json({ message: 'Usuario actualizado', data: userUpdate });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const obtenerUsuarios = async (req, res) => {
  try {
    const { page = 1, search = '', estatus = '' } = req.query;
    const { despacho } = req.params;

    if (!despacho) {
      return res.status(400).json({ message: 'Falta el despacho' });
    }

    const limit = 10;

    const options = {
      page,
      limit,
      sort: {
        estatus: 1,
        nombre: 1
      },
      populate: 'tipoUsuario'
    };

    const query = {
      despacho,
      tipo: 'despacho'
    };

    if (estatus) {
      query.estatus = estatus;
    }

    if (search) {
      query.$or = [
        { nombre: { $regex: search, $options: 'i' } },
        { email: { $regex: search, $options: 'i' } },
        { telefono: { $regex: search, $options: 'i' } },
        { apellidoPaterno: { $regex: search, $options: 'i' } },
        { apellidoMaterno: { $regex: search, $options: 'i' } }
      ];
    }

    const usuarios = await UsuariosModel.paginate(query, options);

    usuarios.docs = usuarios.docs.map((user) => {
      if (user.foto) {
        user.foto = buildFileUri(`despachos/${despacho}/usuarios/${user.foto}`);
      } else {
        user.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
      }

      return user;
    });

    res.status(200).json({ usuarios });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const obtenerUsuariosSinPaginar = async (req, res) => {
  const { despacho } = req.params;

  if (!despacho) {
    return res.status(400).json({ message: 'Falta el despacho' });
  }

  const usuarios = await UsuariosModel.find({
    despacho,
    tipo: 'despacho',
    estatus: 'Activo'
  }).select('nombre apellidoPaterno apellidoMaterno foto');

  usuarios.forEach((user) => {
    if (user.foto) {
      user.foto = buildFileUri(`despachos/${despacho}/usuarios/${user.foto}`);
    } else {
      user.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }
  });

  return res.status(200).json({ usuarios });
};

const createUsuarioDespacho = async (req, res) => {
  try {
    const { despacho } = req.params;
    const {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email,
      tipoUsuario,
      fechaNacimiento
    } = req.body;

    if (!despacho) {
      return res.status(400).json({ message: 'Falta el despacho' });
    }

    if (!nombre) {
      return res.status(400).json({ message: 'Falta el nombre' });
    }

    if (!email) {
      return res.status(400).json({ message: 'Falta el correo' });
    }

    if (!tipoUsuario) {
      return res.status(400).json({ message: 'Falta el tipo de usuario' });
    }

    const fechaNacimientoMexico = fechaNacimiento
      ? moment(fechaNacimiento).tz('America/Mexico_City')
      : null;
    const objUser = {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email,
      tipo: 'despacho',
      despacho,
      estatus: 'Activo',
      tipoUsuario,
      fechaNacimiento: fechaNacimientoMexico
    };

    const findEmailExist = await UsuariosModel.findOne({
      email
    });

    if (findEmailExist) {
      return res.status(400).json({ message: 'El correo ya existe' });
    }

    const passwordSin = generatePassword();
    const passwordEncrypt = encriptar(passwordSin);
    objUser.password = passwordEncrypt;
    const userSave = await UsuariosModel.create(objUser);

    // enviar correo con la contraseña
    const htmlRegistro = RegistroUsuarioHTML(
      nombre,
      email,
      passwordSin,
      `${DESPACHO_APP}/login`
    );
    sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', email);

    sendWhatsappRecovery({
      to: telefono,
      nombre,
      correo: email,
      password: passwordSin,
      url: DESPACHO_APP
    });

    res.status(201).json({ message: 'Usuario creado', data: userSave });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const obtenerUsuario = async (req, res) => {
  try {
    const { id } = req.params;
    const user = await UsuariosModel.findById(id);
    if (!user) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }
    if (user.foto) {
      user.foto = buildFileUri(
        `despachos/${user?.despacho?._id}/usuarios/${user.foto}`
      );
    } else {
      user.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }

    user.password = desencriptar(user.password);

    res.status(200).json({ message: 'Usuario encontrado', data: user });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const updateUsuario = async (req, res) => {
  try {
    const { id } = req.params;
    const {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email,
      tipoUsuario,
      password,
      estatus,
      despacho
    } = req.body;

    let foto = null;

    if (req?.files?.foto) {
      foto = await uploadFile(req.files.foto, `despachos/${despacho}/usuarios`);
    }

    const objUser = {
      nombre,
      apellidoPaterno,
      apellidoMaterno,
      telefono,
      email,
      tipoUsuario,
      estatus,
      password: encriptar(password)
    };

    if (foto) {
      const user = await UsuariosModel.findById(id).select('foto');

      if (user.foto) {
        deleteFileS3(`despachos/${despacho}/usuarios/${user.foto}`);
      }

      objUser.foto = foto.fileName;
    }

    const objAntes = await UsuariosModel.findByIdAndUpdate(id, objUser);

    if (!objAntes) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    res.status(200).json({ message: 'Usuario actualizado' });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const deleteUsuario = async (req, res) => {
  try {
    const { id } = req.params;
    const user = await UsuariosModel.findById(id);

    if (!user) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    user.estatus = 'Inactivo';
    await user.save();

    // await UsuariosModel.findByIdAndDelete(id);
    res.status(200).json({ message: 'Se modifico el estatus del usuario' });
  } catch (error) {
    res.status(404).json({ message: error.message, line_error: error.stack });
  }
};

const cargaUsuario = async (req, res) => {
  try {
    const { usuario } = req.params;

    if (!usuario) {
      return res.status(400).json({ message: 'Falta el usuario' });
    }

    const userFind = await UsuariosModel.findOne({ _id: usuario }).populate(
      'despacho'
    );

    console.log('entro acá');
    const fehcaMexico = moment().tz('America/Mexico_City').format();
    userFind.ultimoAcceso = fehcaMexico;
    await userFind.save();

    if (!userFind) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    userFind.password = desencriptar(userFind.password);

    if (userFind.foto) {
      userFind.foto = buildFileUri(
        `despachos/${userFind?.despacho?._id}/usuarios/${userFind.foto}`
      );
    } else {
      userFind.foto = buildFileUri('default/icono_usuario_100x100_04.jpg');
    }

    if (userFind.despacho && userFind.despacho.logo) {
      userFind.despacho.logo = `${APP_URL}/uploads/despachos/${userFind.despacho.logo}`;
    }

    res.status(200).json({ message: 'Usuario logeado', data: userFind });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const obtenerInformacioDistribuidor = async (req, res) => {
  const { id } = req.params;
  try {
    const [usuario, findCuenta] = await Promise.all([
      UsuariosModel.findById(id).select(
        'nombre apellidoPaterno apellidoMaterno email telefono foto comprobantes despacho rfc'
      ),
      CuentasBancariasModel.findOne({ usuario: id })
    ]);

    if (!usuario) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    if (usuario?.comprobantes?.constancia) {
      usuario.comprobantes.constancia = buildFileUri(
        `distribuidores/${usuario?._id}/comprobantes/${usuario.comprobantes.constancia}`
      );
    }

    if (usuario?.comprobantes?.ine?.frontal) {
      usuario.comprobantes.ine.frontal = buildFileUri(
        `distribuidores/${usuario?._id}/comprobantes/${usuario.comprobantes.ine.frontal}`
      );
    }

    if (usuario?.comprobantes?.ine?.trasera) {
      usuario.comprobantes.ine.trasera = buildFileUri(
        `distribuidores/${usuario?._id}/comprobantes/${usuario.comprobantes.ine.trasera}`
      );
    }

    if (usuario?.comprobantes?.domicilio) {
      usuario.comprobantes.domicilio = buildFileUri(
        `distribuidores/${usuario?._id}/comprobantes/${usuario.comprobantes.domicilio}`
      );
    }

    res.status(200).json({
      usuario,
      cuenta: findCuenta
    });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const actualizarRetirarFondos = async (req, res) => {
  const { id } = req.params;
  const constancia = req?.files?.constancia || null;
  const ineFrontal = req?.files?.ineFrontal || null;
  const ineTrasera = req?.files?.ineTrasera || null;
  const domicilio = req?.files?.domicilio || null;
  const { rfc, tipoTransferencia, numero, tarjeta, banco, titular } = req.body;

  try {
    const [usuario, findCuenta] = await Promise.all([
      UsuariosModel.findById(id),
      CuentasBancariasModel.findOne({ usuario: id })
    ]);

    if (!usuario) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    if (constancia) {
      if (usuario.comprobantes.constancia) {
        deleteFileS3(
          `distribuidores/${usuario._id}/comprobantes/${usuario.comprobantes.constancia}`
        );
      }
      const constanciaFile = await uploadFile(
        constancia,
        `distribuidores/${usuario._id}/comprobantes`
      );
      usuario.comprobantes.constancia = constanciaFile.fileName;
    }

    if (ineFrontal) {
      if (usuario.comprobantes.ine.frontal) {
        deleteFileS3(
          `distribuidores/${usuario._id}/comprobantes/${usuario.comprobantes.ine.frontal}`
        );
      }
      const ineFrontalFile = await uploadFile(
        ineFrontal,
        `distribuidores/${usuario._id}/comprobantes`
      );
      usuario.comprobantes.ine.frontal = ineFrontalFile.fileName;
    }

    if (ineTrasera) {
      if (usuario.comprobantes.ine.trasera) {
        deleteFileS3(
          `distribuidores/${usuario._id}/comprobantes/${usuario.comprobantes.ine.trasera}`
        );
      }
      const ineTraseraFile = await uploadFile(
        ineTrasera,
        `distribuidores/${usuario._id}/comprobantes`
      );
      console.log(ineTraseraFile);
      usuario.comprobantes.ine.trasera = ineTraseraFile.fileName;
    }

    if (domicilio) {
      if (usuario.comprobantes.domicilio) {
        deleteFileS3(
          `distribuidores/${usuario._id}/comprobantes/${usuario.comprobantes.domicilio}`
        );
      }
      const domicilioFile = await uploadFile(
        domicilio,
        `distribuidores/${usuario._id}/comprobantes`
      );
      usuario.comprobantes.domicilio = domicilioFile.fileName;
    }
    usuario.rfc = rfc;

    usuario.save();

    if (findCuenta) {
      findCuenta.tipo = tipoTransferencia;
      findCuenta.banco = banco;
      findCuenta.titular = titular;

      if (tipoTransferencia === 'Débito') {
        findCuenta.banco = banco;
        findCuenta.numero = tarjeta;
      }

      if (tipoTransferencia === 'Clabe') {
        findCuenta.numero = numero;
        findCuenta.banco = null;
      }

      await findCuenta.save();
    } else {
      const cuentaNueva = {
        usuario: id,
        tipo: tipoTransferencia,
        titular
      };

      if (tipoTransferencia === 'Débito') {
        cuentaNueva.banco = banco;
        cuentaNueva.numero = tarjeta;
      }

      if (tipoTransferencia === 'Clabe') {
        cuentaNueva.numero = numero;
      }

      await CuentasBancariasModel.create(cuentaNueva);
    }

    res.status(200).json({ message: 'Comprobantes actualizados' });
  } catch (error) {
    res.status(404).json({ message: error.message });
  }
};

const findDistribuidorForClave = async (req, res) => {
  try {
    const { clave } = req.params;

    if (!clave) {
      return res.status(400).json({ message: 'Falta la clave' });
    }

    const findDistribuidor = await UsuariosModel.findOne({
      clave,
      tipo: 'distribuidor',
      // producto si es diferente null
      'licencia.producto': { $ne: null },
      estatus: 'Activo'
    }).select(
      'nombre apellidoPaterno apellidoMaterno email telefono foto licencia clave'
    );

    if (!findDistribuidor) {
      return res.status(404).json({ message: 'Distribuidor no encontrado' });
    }

    // licencia.vigencias que aun esta activa
    if (findDistribuidor?.licencia) {
      const fechaMexico = moment().tz('America/Mexico_City');
      if (
        findDistribuidor &&
        findDistribuidor.licencia.vigencia > fechaMexico
      ) {
        if (findDistribuidor.foto) {
          findDistribuidor.foto = buildFileUri(
            `usuarios/${findDistribuidor.foto}`
          );
        } else {
          findDistribuidor.foto = buildFileUri(
            'default/icono_usuario_100x100_04.jpg'
          );
        }
        return res
          .status(200)
          .json({ message: 'Distribuidor encontrado', data: findDistribuidor });
      } else {
        return res.status(404).json({ message: 'La licencia ha expirado' });
      }
    } else {
      return res.status(404).json({ message: 'Distribuidor no encontrado' });
    }
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};

const findDistribuidorForClaveProspecto = async (req, res) => {
  try {
    const { clave } = req.params;

    if (!clave) {
      return res.status(400).json({ message: 'Falta la clave' });
    }

    const findDistribuidor = await UsuariosModel.findOne({
      clave,
      tipo: 'distribuidor',
      estatus: 'Prospecto'
    }).select(
      'nombre apellidoPaterno apellidoMaterno email telefono foto licencia'
    );

    if (!findDistribuidor) {
      return res.status(404).json({ message: 'Distribuidor no encontrado' });
    }

    return res
      .status(200)
      .json({ message: 'Distribuidor encontrado', data: findDistribuidor });
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};

const generateDistribuidorRegistroProspecto = async (req, res) => {
  const { nombreCompleto, correo, telefono, referidoPor = null } = req.body;

  if (!nombreCompleto) {
    return res.status(400).json({ message: 'Falta el nombre' });
  }

  if (!correo) {
    return res.status(400).json({ message: 'Falta el correo' });
  }

  if (!telefono) {
    return res.status(400).json({ message: 'Falta el telefono' });
  }

  try {
    // buscar si no existe el correo
    const findEmailExist = await UsuariosModel.findOne({
      email: correo
    });

    if (findEmailExist) {
      return res.status(400).json({ message: 'El correo ya existe' });
    }

    const passwordSin = generatePassword();

    const passwordEncrypt = encriptar(passwordSin);

    const findTipoUsuarioDistribuidor = await TipoUsuarioModel.findOne({
      tipo: 'distribuidor'
    }).select('_id');

    let referido = null;

    if (referidoPor) {
      const findDistribuidor = await UsuariosModel.findOne({
        clave: referidoPor
      }).select('_id');

      if (findDistribuidor) {
        referido = findDistribuidor._id;
      }
    }

    const objUser = {
      nombre: nombreCompleto,
      email: correo,
      telefono,
      password: passwordEncrypt,
      tipo: 'distribuidor',
      estatus: 'Prospecto',
      tipoUsuario: findTipoUsuarioDistribuidor._id,
      licencia: {
        referidoPor: referido,
        comision: 0
      }
    };

    const userSave = await UsuariosModel.create(objUser);

    // sendWhatsappRecovery({ to: '2213425514', correo, password: passwordSin, url: DISTRIBUIDOR_APP });

    res.status(201).json({ message: 'Usuario creado', data: userSave });
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};
const ConvertirProspectoDistribuidor = async (req, res) => {
  const { _id, producto, tipoReferencia, referencia } = req.body;

  if (!_id) {
    return res.status(400).json({ message: 'Falta el id del usuario' });
  }

  if (!producto) {
    return res.status(400).json({ message: 'Falta el producto' });
  }
  if (!tipoReferencia) {
    return res.status(400).json({ message: 'Falta el producto' });
  }
  if (!referencia) {
    return res.status(400).json({ message: 'Falta la referencia' });
  }

  try {
    const findUsuario = await UsuariosModel.findById(_id).populate(
      'licencia.referidoPor'
    );

    if (!findUsuario) {
      return res.status(404).json({ message: 'Usuario no encontrado' });
    }

    const findProducto = await productoModel.findById(producto);

    if (!findProducto) {
      return res.status(404).json({ message: 'Producto no encontrado' });
    }

    const fechaMexico = moment().tz('America/Mexico_City');

    findUsuario.licencia = {
      ...findUsuario.licencia, // Mantiene las propiedades existentes en licencia
      producto, // Sobrescribe el producto
      vigencia: fechaMexico.add(findProducto.cantidad, 'years'), // Actualiza la vigencia
      comision: findProducto.utilidad // Actualiza la comisión
    };

    findUsuario.estatus = 'Activo';

    await findUsuario.save();

    // falta mandar su contrato

    const urlAjunto = buildFileUri(`productos/${findProducto.adjunto}`);

    const passwordSin = desencriptar(findUsuario.password);

    const htmlRegistro = RegistroUsuarioHTML(
      findUsuario.nombre,
      findUsuario.email,
      passwordSin,
      `${DISTRIBUIDOR_APP}/login`
    );

    const files = [
      {
        filename: 'Contrato.pdf',
        path: urlAjunto
      }
    ];

    sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', findUsuario.email);
    sendMail(htmlRegistro, 'Contrato Distribuidor', findUsuario.email, files);

    sendWhatsappRecovery({
      to: findUsuario.telefono,
      correo: findUsuario.email,
      password: passwordSin,
      url: DISTRIBUIDOR_APP
    });

    // venta de producto

    const findDistribuidor = await UsuariosModel.findById(
      findUsuario.licencia.referidoPor
    ).select('licencia');

    let licencia;
    if (findDistribuidor) {
      licencia = findDistribuidor.licencia;
    }

    const newVenta = new Ventas({
      despacho: null,
      usuario: findUsuario._id,
      producto: findProducto._id,
      importe: findProducto.precio,
      tipoReferencia,
      referencia,
      estatus: 'Pagado',
      distribuidor: findDistribuidor
        ? {
          comision: licencia.comision,
          distribuidor: findDistribuidor._id,
          importe: (findProducto.precio * licencia.comision) / 100
        }
        : {
          comision: 0,
          distribuidor: null,
          importe: 0
        }
    });

    await VentasSchema.create(newVenta);

    res.status(201).json({ message: 'Usuario creado', data: findUsuario });
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};

const generateDistribuidorRegistroPruebas = async (req, res) => {
  const { nombreCompleto, correo, telefono } = req.body;

  if (!nombreCompleto) {
    return res.status(400).json({ message: 'Falta el nombre' });
  }

  if (!correo) {
    return res.status(400).json({ message: 'Falta el correo' });
  }

  if (!telefono) {
    return res.status(400).json({ message: 'Falta el telefono' });
  }

  try {
    // buscar si no existe el correo
    // const findEmailExist = await UsuariosModel.findOne({
    //   email: correo
    // });

    // if (findEmailExist) {
    //   return res.status(400).json({ message: 'El correo ya existe' });
    // }

    const passwordSin = generatePassword();

    const passwordEncrypt = encriptar(passwordSin);

    const findTipoUsuarioDistribuidor = await TipoUsuarioModel.findOne({
      tipo: 'distribuidor'
    }).select('_id');

    // const producto20 = '66c7b0deb8586f9830adb34c';
    const producto30 = '66c7c834b8586f9830adb34d';
    // const producto40 = '66c7c8f8b8586f9830adb34e';

    // const vigencia1 = moment().tz('America/Mexico_City').add(1, 'years');
    const vigencia2 = moment().tz('America/Mexico_City').add(2, 'years');
    // const vigencia3 = moment().tz('America/Mexico_City').add(3, 'years');

    const objUser = {
      nombre: nombreCompleto,
      email: correo,
      telefono,
      password: passwordEncrypt,
      tipo: 'distribuidor',
      estatus: 'Activo',
      tipoUsuario: findTipoUsuarioDistribuidor._id,
      licencia: {
        producto: producto30,
        vigencia: vigencia2,
        comision: 30
      }
    };

    const userSave = await UsuariosModel.create(objUser);

    // const htmlRegistro = RegistroUsuarioHTML(nombreCompleto, correo, passwordSin, `${DISTRIBUIDOR_APP}/login`);

    // sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);

    sendWhatsappRecovery({
      to: '2213425514',
      correo,
      password: passwordSin,
      url: DISTRIBUIDOR_APP
    });

    res.status(201).json({ message: 'Usuario creado', data: userSave });
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};

const generateDistribuidorRegistro = async (req, res) => {
  const { nombreCompleto, correo, telefono, producto } = req.body;

  if (!nombreCompleto) {
    return res.status(400).json({ message: 'Falta el nombre' });
  }

  if (!correo) {
    return res.status(400).json({ message: 'Falta el correo' });
  }

  if (!telefono) {
    return res.status(400).json({ message: 'Falta el telefono' });
  }

  if (!producto) {
    return res.status(400).json({ message: 'Falta el producto' });
  }

  try {
    // buscar si no existe el correo
    const findEmailExist = await UsuariosModel.findOne({
      email: correo
    });

    if (findEmailExist) {
      return res.status(400).json({ message: 'El correo ya existe' });
    }

    const passwordSin = generatePassword();

    const passwordEncrypt = encriptar(passwordSin);

    const findTipoUsuarioDistribuidor = await TipoUsuarioModel.findOne({
      tipo: 'distribuidor'
    }).select('_id');

    const findProducto = await productoModel.findById(producto);

    if (!findProducto) {
      return res.status(404).json({ message: 'Producto no encontrado' });
    }

    const fechaMexico = moment().tz('America/Mexico_City');
    const objUser = {
      nombre: nombreCompleto,
      email: correo,
      telefono,
      password: passwordEncrypt,
      tipo: 'distribuidor',
      estatus: 'Prospecto',
      tipoUsuario: findTipoUsuarioDistribuidor._id,
      licencia: {
        producto,
        vigencia: fechaMexico.add(findProducto.cantidad, 'years'),
        comision: findProducto.utilidad
      }
    };

    const userSave = await UsuariosModel.create(objUser);

    sendWhatsappRecovery({
      to: telefono,
      correo,
      password: passwordSin,
      url: DISTRIBUIDOR_APP
    });

    const htmlRegistro = RegistroUsuarioHTML(
      nombreCompleto,
      correo,
      passwordSin,
      `${DISTRIBUIDOR_APP}/login`
    );

    sendMail(htmlRegistro, 'IUSFORM: Envió Accesos', correo);

    res.status(201).json({ message: 'Usuario creado', data: userSave });
  } catch (error) {
    console.log(error);
    return res.status(404).json({ message: error.message });
  }
};

module.exports = {
  login,
  register,
  rememberPassword,
  actualizarUsuario,
  obtenerUsuarios,
  createUsuarioDespacho,
  obtenerUsuario,
  updateUsuario,
  deleteUsuario,
  cargaUsuario,
  loginCliente,
  obtenerUsuariosSinPaginar,
  obtenerInformacioDistribuidor,
  actualizarRetirarFondos,
  findDistribuidorForClave,
  generateDistribuidorRegistroProspecto,
  findDistribuidorForClaveProspecto,
  ConvertirProspectoDistribuidor,
  generateDistribuidorRegistro,
  generateDistribuidorRegistroPruebas
};
